package com.gic.store.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.AuthModeEnum;
import com.gic.auth.constant.AuthorizationStatusEnum;
import com.gic.auth.service.UnionEnterpriseApiService;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.commons.util.ToolUtil;
import com.gic.enterprise.constant.union.UnionEnterpriseAuthDetailStatusEnum;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.UnionEnterpriseAuthResDetailApiService;
import com.gic.mq.sdk.GicMQClient;
import com.gic.store.constant.StoreGroupConstant;
import com.gic.store.constant.StoreOwnTypeEnum;
import com.gic.store.dto.StoreDTO;
import com.gic.store.dto.StoreSearchDTO;
import com.gic.store.dto.StoreWidgetDTO;
import com.gic.store.entity.TabAuthStoreLog;
import com.gic.store.entity.TabStoreGroup;
import com.gic.store.service.*;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 
 * @ClassName: StoreAuthorizationApiServiceImpl
 * @Description:  
 * @author zhiwj
 * @date 2020-09-09 17:53
 */
@Service("storeAuthorizationApiService")
public class StoreAuthorizationApiServiceImpl implements StoreAuthorizationApiService {
    private final static Logger log = LogManager.getLogger(StoreAuthorizationApiServiceImpl.class);
    private final static int PAGE_SIZE = 1000;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private StoreStrategyService storeStrategyService;
    @Autowired
    private StoreGroupService storeGroupService;
    @Autowired
    private StoreService storeService;
    @Autowired
    private UnionEnterpriseApiService unionEnterpriseApiService;
    @Autowired
    private UnionEnterpriseAuthResDetailApiService unionEnterpriseAuthResDetailApiService;
    @Autowired
    private StoreRegionApiService storeRegionApiService;
    @Autowired
    private StoreDictApiService storeDictApiService;
    @Autowired
    private AuthStoreLogService authStoreLogService;
    @Autowired
    private StoreBrandService storeBrandService;
    @Autowired
    private StoreRegionRelService storeRegionRelService;

    @Override
    public ServiceResponse<Void> authStore(Integer storeResourceId, Integer unionEnterpriseId, Integer ownEnterpriseId) {
        StoreWidgetDTO storeWidgetDTO = this.storeWidgetApiService.getStoreWidget(storeResourceId).getResult();
        if(storeWidgetDTO == null){
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店资源Id不存在");
        }
        String key = ToolUtil.randomUUID();
        Integer authMode = storeWidgetDTO.getAuthMode();
        if(authMode.equals(AuthModeEnum.NO.getCode())){
            this.unionEnterpriseAuthResDetailApiService.unionForStore(UnionEnterpriseAuthDetailStatusEnum.HAS_AUTH.getCode(), ownEnterpriseId, unionEnterpriseId, key, "门店资源不授权");
            return ServiceResponse.success();
        }
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(ownEnterpriseId);
        storeSearchDTO.setSearchJson(storeWidgetDTO.getSearchParam());
        Long count = this.storeApiService.queryStoreCountFromEs(storeSearchDTO).getResult();
        if(count < 0){
            this.unionEnterpriseAuthResDetailApiService.unionForStore(UnionEnterpriseAuthDetailStatusEnum.HAS_AUTH.getCode(), ownEnterpriseId, unionEnterpriseId, key, "授权成功");
        } else {
            this.unionEnterpriseAuthResDetailApiService.unionForStore(UnionEnterpriseAuthDetailStatusEnum.RESOURCE_DEALING.getCode(), ownEnterpriseId, unionEnterpriseId, key, "资源授权中");
        }
        this.queryStoreByPage(count, storeSearchDTO, unionEnterpriseId, ownEnterpriseId, key, 0);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> reAuthStore(String key) {
        List<TabAuthStoreLog> list = this.authStoreLogService.listFailedStore(key);
        StringBuilder storeInfoIds = new StringBuilder();
        if(CollectionUtils.isNotEmpty(list)){
            Integer fromEnterpriseId = list.get(0).getFromEnterpriseId();
            Integer toEnterpriseId = list.get(0).getToEnterpriseId();
            for(TabAuthStoreLog log : list){
                storeInfoIds.append(log.getStoreInfoId()).append(",");
            }
            StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
            storeSearchDTO.setEnterpriseId(toEnterpriseId);
            storeSearchDTO.setStoreInfoIds(storeInfoIds.toString());
            this.queryStoreByPage(Long.valueOf(list.size()+""), storeSearchDTO, toEnterpriseId, fromEnterpriseId, key, 1);
        }
        return ServiceResponse.success();
    }


    @Override
    public ServiceResponse<Void> authorizationSingleStore(String params){
        log.info("authorizationSingleStore:{}", params);
        JSONObject json = JSON.parseObject(params);
        StoreDTO storeDTO = JSON.toJavaObject(json.getJSONObject("store"), StoreDTO.class);
        Integer toEnterpriseId = json.getInteger("toEnterpriseId");
        Integer fromEnterpriseId = json.getInteger("enterpriseId");
        Integer isEnd = json.getInteger("isEnd");
        String key = json.getString("key");
        storeDTO.setEnterpriseId(toEnterpriseId);
        storeDTO.setFromEnterpriseId(fromEnterpriseId);
        int logId = this.authStoreLogService.saveAuthStoreLog(storeDTO.getStoreInfoId(), fromEnterpriseId, toEnterpriseId, key);
        this.saveStore(storeDTO, logId);
        log.info("是否最后一条：{}", isEnd);
        if(isEnd == 1){
            int count = this.authStoreLogService.getCount(key, 2);
            int total = this.authStoreLogService.getCount(key, null);
            log.info("失败条数：{}", count);
            if(count > 0){
                this.unionEnterpriseAuthResDetailApiService.unionForStore(UnionEnterpriseAuthDetailStatusEnum.AUTH_PART_FAIL.getCode(), fromEnterpriseId, toEnterpriseId, key, "成功授权"+(total-count)+"条数据，失败"+count+"条");
            }else {
                this.unionEnterpriseAuthResDetailApiService.unionForStore(UnionEnterpriseAuthDetailStatusEnum.HAS_AUTH.getCode(), fromEnterpriseId, toEnterpriseId, key, "成功授权");
            }
        }
        return null;
    }

    @Override
    public ServiceResponse<Void> saveStore(StoreDTO storeDTO, Integer logId){
        String groupId = storeStrategyService.isHitStrategy(storeDTO, StoreGroupConstant.STORE_GROUP_STRATEGY_TYPE);
        if (StringUtils.isBlank(groupId)) {
            TabStoreGroup group = storeGroupService.selectUnGroupedStore(storeDTO.getEnterpriseId());
            if(group != null){
                groupId = group.getStoreGroupId()+"";
                storeDTO.setStoreGroupId(Integer.valueOf(groupId));
            }
        }
        storeDTO.setIsEditStoreGroup(0);
        storeDTO.setOwnType(StoreOwnTypeEnum.OTHER.getCode());
        log.info("授权门店信息：{}", JSON.toJSONString(storeDTO));
        int i = this.storeService.authAddStore(storeDTO);
        log.info("{}，门店授权结果：{}", storeDTO.getStoreInfoId(), i);
        if(i == 0 && logId != 0){
            this.authStoreLogService.updateStatus(logId, 2);
        }else {
            this.authStoreLogService.updateStatus(logId, 1);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> saveStore(StoreDTO storeDTO) {
        return this.saveStore(storeDTO, 0);
    }

    @Override
    public ServiceResponse<Void> cancelStore(Integer storeId, Integer enterpriseId){
        List<Integer> list = new ArrayList<>();
        list.add(storeId);
        this.storeService.cancelAuthStore(list, enterpriseId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> cancelStoreAll(Integer unionEnterpriseId, Integer ownEnterpriseId) {
        /**门店授权取消**/
        int storeCancelCount = this.storeService.cancelAuthStoreAll(ownEnterpriseId, unionEnterpriseId);
        log.info("门店取消授权数量:{}", storeCancelCount);
        /**门店域取消**/
        int count = this.storeRegionRelService.cancelRegionAuth(ownEnterpriseId, unionEnterpriseId);
        log.info("门店域取消授权数量：{}", count);
        /***门店品牌取消授权**/
        int i = this.storeBrandService.cancelBrandAuth(ownEnterpriseId, unionEnterpriseId);
        log.info("门店品牌取消授权数量：{}", i);
        this.unionEnterpriseAuthResDetailApiService.relieveForStore(UnionEnterpriseAuthDetailStatusEnum.HAS_RELIEVE.getCode(), ownEnterpriseId, unionEnterpriseId);
        return ServiceResponse.success();
    }

    private void queryStoreByPage(Long count, StoreSearchDTO storeSearchDTO, Integer unionEnterpriseId, Integer ownEnterpriseId, String key, Integer authType){
        int pages = count/ PAGE_SIZE + count % PAGE_SIZE >0 ? 1 : 0;
        /**需要共享的域*/
        Set<Integer> regionSet = new HashSet<>();
        /**需要共享的门店类型****/
        Set<String> storeTypeSet = new HashSet<>();
        /**需要共享的门店状态****/
        Set<String> storeStatusSet = new HashSet<>();
        /**需要共享的门店品牌****/
        Set<Integer> storeBrandIdSet = new HashSet<>();
        /**获取当前商户下的共享门店**/
        List<Integer> existStoreInfoIdList = this.storeService.listShareStoreId(ownEnterpriseId);
        Set<Integer> storeInfoIdSet = new HashSet<>(existStoreInfoIdList);
        int total = 0;
        for(int page = 1; page <= pages; page++){
            Page<StoreDTO> storeDTOPage = this.storeApiService.listStore(storeSearchDTO, page, PAGE_SIZE).getResult();
            if(storeDTOPage == null){
                break;
            }
            List<StoreDTO> list = storeDTOPage.getResult();
            if(CollectionUtils.isEmpty(list)){
                break;
            }
            int isEnd = 0;
            for(int k=0; k< list.size(); k++){
                StoreDTO storeDTO = list.get(k);
                regionSet.add(storeDTO.getRegionId());
                storeTypeSet.add(storeDTO.getStoreType() == null ? null : storeDTO.getStoreType().toString());
                storeStatusSet.add(storeDTO.getStatus() == null ? null : storeDTO.getStatus().toString());
                if(CollectionUtils.isNotEmpty(storeDTO.getStoreBrandIdList())){
                    storeBrandIdSet.addAll(storeDTO.getStoreBrandIdList());
                }
                log.info("pages:{},page:{},k:{}", pages,page, k);
                if(page == pages && k == list.size()-1) {
                    isEnd = 1;
                }
                total++;
                this.sendDataToMq(storeDTO, unionEnterpriseId, isEnd, key, total);
                /**共享门店不能再共享 所以需要剔除***/
                if(storeInfoIdSet.contains(storeDTO.getStoreInfoId())){
                    storeInfoIdSet.remove(storeDTO.getStoreInfoId());
                }
            }
        }
        this.authRegionStatusStoreType(regionSet, storeStatusSet, storeTypeSet, storeInfoIdSet, storeBrandIdSet, ownEnterpriseId, unionEnterpriseId, authType);
    }

    private void sendDataToMq(StoreDTO storeDTO, Integer toEnterpriseId, int isEnd, String key, int total){
        Map<String, Object> params = new HashMap<>();
        params.put("enterpriseId", storeDTO.getEnterpriseId());
        params.put("store", storeDTO);
        params.put("toEnterpriseId", toEnterpriseId);
        params.put("isEnd", isEnd);
        params.put("key", key);
        params.put("total", total);
        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            client.sendMessage("storeAuthorizationMq", JSON.toJSONString(params));
        } catch (Exception e) {
            log.warn("exception", e);
        }
    }

    private void authRegionStatusStoreType(Set<Integer> regionSet, Set<String> statusSet, Set<String> storeTypeSet,
                                           Set<Integer> storeInfoIdSet, Set<Integer> storeBrandIdSet, Integer enterpriseId, Integer toEnterpriseId, Integer authType){
        this.storeRegionApiService.authorizeRegionToOther(enterpriseId, toEnterpriseId, new ArrayList<>(regionSet), authType);
        this.storeDictApiService.saveStoreStatusAndStoreType(enterpriseId, statusSet.toArray(new String[0]), storeTypeSet.toArray(new String[0]));
        this.storeBrandService.saveStoreBrandRef(toEnterpriseId, new ArrayList<>(storeBrandIdSet), 2, enterpriseId);
        if(CollectionUtils.isNotEmpty(storeInfoIdSet)){
            this.storeService.cancelAuthStore(new ArrayList<>(storeInfoIdSet), toEnterpriseId);
        }
    }
}
